@@ -26,13 +26,18 @@ std::set<std::string> keep;
 void usage(char **argv) {
 	fprintf(stderr, "Usage: %s -o newtile.pbf.gz tile.pbf.gz oz/ox/oy nz/nx/ny\n", argv[0]);
 	fprintf(stderr, "to create tile nz/nx/ny from tile oz/ox/oy\n");
+	fprintf(stderr, "Usage: %s -o newtile.pbf.gz -t nz/nx/ny tile.pbf.gz oz/ox/oy tile2.pbf.gz oz2/ox2/oy2\n", argv[0]);
+	fprintf(stderr, "to create tile nz/nx/ny from tiles oz/ox/oy and oz2/ox2/oy2\n");
 	exit(EXIT_FAILURE);
 }
 
 int main(int argc, char **argv) {
 	int i;
+	const char *outtile = NULL;
 	const char *outfile = NULL;
 
+	std::vector<input_tile> sources;
+
 	struct option long_options[] = {
 		{"include", required_argument, 0, 'y'},
 		{"full-detail", required_argument, 0, 'd'},
@@ -43,6 +48,7 @@ int main(int argc, char **argv) {
 		{"preserve-input-order", no_argument, 0, 'o' & 0x1F},
 		{"accumulate-attribute", required_argument, 0, 'E'},
 		{"unidecode-data", required_argument, 0, 'u' & 0x1F},
+		{"source-tile", required_argument, 0, 't'},
 
 		{0, 0, 0, 0},
 	};
@@ -97,53 +103,71 @@ int main(int argc, char **argv) {
 			unidecode_data = read_unidecode(optarg);
 			break;
 
+		case 't':
+			outtile = optarg;
+			break;
+
 		default:
 			fprintf(stderr, "Unrecognized flag -%c\n", i);
 			usage(argv);
 		}
 	}
 
-	if (argc - optind != 3) {
-		usage(argv);
-	}
-
-	if (outfile == NULL) {
-		usage(argv);
-	}
-
-	const char *infile = argv[optind + 0];
-
-	int oz, ox, oy;
-	if (sscanf(argv[optind + 1], "%d/%d/%d", &oz, &ox, &oy) != 3) {
-		fprintf(stderr, "%s: not in z/x/y form\n", argv[optind + 1]);
-		usage(argv);
-	}
-
+	std::vector<input_tile> its;
 	int nz, nx, ny;
-	if (sscanf(argv[optind + 2], "%d/%d/%d", &nz, &nx, &ny) != 3) {
-		fprintf(stderr, "%s: not in z/x/y form\n", argv[optind + 2]);
-		usage(argv);
-	}
 
-	std::string tile;
-	char buf[1000];
-	int len;
+	if (outtile == NULL) {	// single input
+		if (argc - optind != 3) {
+			fprintf(stderr, "Wrong number of arguments\n");
+			usage(argv);
+		}
 
-	FILE *f = fopen(infile, "rb");
-	if (f == NULL) {
-		perror(infile);
-		exit(EXIT_FAILURE);
-	}
+		const char *infile = argv[optind + 0];
 
-	while ((len = fread(buf, sizeof(char), 1000, f)) > 0) {
-		tile.append(std::string(buf, len));
-	}
-	fclose(f);
+		int oz, ox, oy;
+		if (sscanf(argv[optind + 1], "%d/%d/%d", &oz, &ox, &oy) != 3) {
+			fprintf(stderr, "%s: not in z/x/y form\n", argv[optind + 1]);
+			usage(argv);
+		}
 
-	f = fopen(outfile, "wb");
-	if (f == NULL) {
-		perror(outfile);
-		exit(EXIT_FAILURE);
+		if (sscanf(argv[optind + 2], "%d/%d/%d", &nz, &nx, &ny) != 3) {
+			fprintf(stderr, "%s: not in z/x/y form\n", argv[optind + 2]);
+			usage(argv);
+		}
+
+		input_tile s;
+		s.tile = infile;
+		s.z = oz;
+		s.x = ox;
+		s.y = oy;
+
+		sources.push_back(s);
+	} else {  // multiple inputs
+		if ((argc - optind) % 2 != 0) {
+			fprintf(stderr, "Unpaired arguments\n");
+			usage(argv);
+		}
+
+		if (sscanf(outtile, "%d/%d/%d", &nz, &nx, &ny) != 3) {
+			fprintf(stderr, "%s: not in z/x/y form\n", outtile);
+			usage(argv);
+		}
+
+		for (i = optind; i + 1 < argc; i += 2) {
+			int oz, ox, oy;
+			if (sscanf(argv[i + 1], "%d/%d/%d", &oz, &ox, &oy) != 3) {
+				fprintf(stderr, "%s: not in z/x/y form\n", argv[i + 1]);
+				usage(argv);
+			}
+
+			input_tile s;
+			s.tile = argv[i];
+			s.z = oz;
+			s.x = ox;
+			s.y = oy;
+
+			sources.push_back(s);
+		}
 	}
 
 	json_object *json_filter = NULL;
@@ -151,7 +175,35 @@ int main(int argc, char **argv) {
 		json_filter = parse_filter(filter.c_str());
 	}
 
-	std::string out = overzoom(tile, oz, ox, oy, nz, nx, ny, detail, buffer, keep, true, NULL, demultiply, json_filter, preserve_input_order, attribute_accum, unidecode_data);
+	for (auto const &s : sources) {
+		std::string tile;
+		char buf[1000];
+		int len;
+
+		FILE *f = fopen(s.tile.c_str(), "rb");
+		if (f == NULL) {
+			perror(s.tile.c_str());
+			exit(EXIT_FAILURE);
+		}
+
+		while ((len = fread(buf, sizeof(char), 1000, f)) > 0) {
+			tile.append(std::string(buf, len));
+		}
+		fclose(f);
+
+		input_tile t = s;
+		t.tile = std::move(tile);
+		its.push_back(std::move(t));
+	}
+
+	std::string out = overzoom(its, nz, nx, ny, detail, buffer, keep, true, NULL, demultiply, json_filter, preserve_input_order, attribute_accum, unidecode_data);
+
+	FILE *f = fopen(outfile, "wb");
+	if (f == NULL) {
+		perror(outfile);
+		exit(EXIT_FAILURE);
+	}
+
 	fwrite(out.c_str(), sizeof(char), out.size(), f);
 	fclose(f);
 
